@@ -94,7 +94,7 @@ python-oauth2 is available on
 """
 
 import json
-from oauth2.client_authenticator import ClientAuthenticator
+from oauth2.client_authenticator import ClientAuthenticator, request_body
 from oauth2.error import OAuthInvalidError, \
     ClientNotFoundError, OAuthInvalidNoRedirectError, UnsupportedGrantError
 from oauth2.web import Request, Response
@@ -102,7 +102,7 @@ from oauth2.tokengenerator import Uuid4
 from oauth2.grant import Scope, AuthorizationCodeGrant, ImplicitGrant, \
     ClientCredentialsGrant, ResourceOwnerGrant, RefreshToken
 
-VERSION = "0.6.0"
+VERSION = "0.7.0"
 
 
 class Provider(object):
@@ -110,12 +110,14 @@ class Provider(object):
     token_path = "/token"
 
     def __init__(self, access_token_store, auth_code_store, client_store,
-                 site_adapter, token_generator, response_class=Response):
+                 site_adapter, token_generator,
+                 client_authentication_source=request_body,
+                 response_class=Response):
         """
         Endpoint of requests to the OAuth 2.0 provider.
 
-        :param access_token_store: An object that implements methods defined by
-                                   :class:`oauth2.store.AccessTokenStore`.
+        :param access_token_store: An object that implements methods defined
+                                   by :class:`oauth2.store.AccessTokenStore`.
         :param auth_code_store: An object that implements methods defined by
                                 :class:`oauth2.store.AuthTokenStore`.
         :param client_store: An object that implements methods defined by
@@ -123,6 +125,9 @@ class Provider(object):
         :param site_adapter: An object that implements methods defined by
                              :class:`oauth2.web.SiteAdapter`.
         :param token_generator: Object to generate unique tokens.
+        :param client_authentication_source: A callable which when executed,
+                                             authenticates a client.
+                                             See :module:`oauth2.client_authenticator`.
         :param response_class: Class of the response object.
                                Defaults to :class:`oauth2.web.Response`.
 
@@ -133,7 +138,8 @@ class Provider(object):
         self.access_token_store = access_token_store
         self.auth_code_store = auth_code_store
         self.client_authenticator = ClientAuthenticator(
-            client_store=client_store)
+            client_store=client_store,
+            source=client_authentication_source)
         self.response_class = response_class
         self.site_adapter = site_adapter
         self.token_generator = token_generator
@@ -146,7 +152,10 @@ class Provider(object):
                       :class:`oauth2.grant.GrantHandlerFactory`
         """
         if hasattr(grant, "expires_in"):
-            self.token_generator.expires_in = grant.expires_in
+            self.token_generator.expires_in[grant.grant_type] = grant.expires_in
+
+        if hasattr(grant, "refresh_expires_in"):
+            self.token_generator.refresh_expires_in = grant.refresh_expires_in
 
         self.grant_types.append(grant)
 
@@ -173,8 +182,6 @@ class Provider(object):
             response.status_code = 400
             return response
         except OAuthInvalidError as err:
-            print(err.error)
-            print(err.explanation)
             response = self.response_class()
             return grant_type.handle_error(error=err, response=response)
         except UnsupportedGrantError:
