@@ -505,7 +505,6 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
             handler.read_validate_params(request_mock)
 
         error = expected.exception
-        print(error.explanation)
 
         self.assertEqual(error.error, "invalid_grant")
         self.assertEqual(error.explanation, "Authorization code has expired")
@@ -566,6 +565,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 10000
 
         response_mock = Mock(spec=Response)
         response_mock.body = None
@@ -646,6 +646,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
             side_effect = AccessTokenNotFound
 
         token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.refresh_expires_in = 10000
         token_generator_mock.create_access_token_data.\
             return_value = token_data
 
@@ -682,6 +683,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 10000
 
         handler = AuthorizationCodeTokenHandler(
             access_token_store=access_token_store_mock,
@@ -719,6 +721,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 10000
 
         handler = AuthorizationCodeTokenHandler(
             access_token_store=access_token_store_mock,
@@ -1073,7 +1076,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
 
         site_adapter_mock.authenticate.assert_called_with(request_mock, {},
                                                           scopes)
-        token_generator_mock.create_access_token_data.assert_called_with()
+        token_generator_mock.create_access_token_data.assert_called_with(ResourceOwnerGrant.grant_type)
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
         self.assertEqual(access_token.grant_type,
@@ -1114,6 +1117,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 1200
 
         handler = ResourceOwnerGrantHandler(
             access_token_store=access_token_store_mock,
@@ -1126,7 +1130,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
 
         site_adapter_mock.authenticate.assert_called_with(request_mock, {},
                                                           scopes)
-        token_generator_mock.create_access_token_data.assert_called_with()
+        token_generator_mock.create_access_token_data.assert_called_with(ResourceOwnerGrant.grant_type)
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
         self.assertEqual(access_token.user_id, user[1])
@@ -1170,7 +1174,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         handler.client = Client(identifier="abc", secret="xyz")
         result = handler.process(Mock(Request), response_mock, {})
 
-        token_generator_mock.create_access_token_data.assert_called_with()
+        token_generator_mock.create_access_token_data.assert_called_with(ResourceOwnerGrant.grant_type)
         response_mock.add_header.assert_has_calls([call("Content-Type",
                                                         "application/json"),
                                                    call("Cache-Control",
@@ -1432,7 +1436,7 @@ class RefreshTokenTestCase(unittest.TestCase):
 
 class RefreshTokenHandlerTestCase(unittest.TestCase):
     @patch("time.time", mock_time)
-    def test_process(self):
+    def test_process_no_reissue(self):
         client_id = "testclient"
         data = {"additional": "data"}
         expires_in = 600
@@ -1451,9 +1455,10 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         scope_handler_mock = Mock(spec=Scope)
         scope_handler_mock.scopes = scopes
 
-        token_generator_mock = Mock(spec=["generate"])
-        token_generator_mock.expires_in = expires_in
-        token_generator_mock.generate.return_value = token
+        token_data = {"access_token": token, "expires_in":expires_in, "token_type": "Bearer", "refresh_token":"gafc"}
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 1200
 
         handler = RefreshTokenHandler(
             access_token_store=access_token_store_mock,
@@ -1462,13 +1467,14 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
             token_generator=token_generator_mock)
         handler.client = Client(identifier=client_id, secret="xyz")
         handler.data = data
+        handler.refresh_grant_type = 'test_grant_type'
 
         result = handler.process(request=Mock(spec=Request),
                                  response=response, environ={})
 
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertEqual(access_token.client_id, client_id)
-        self.assertEqual(access_token.grant_type, "refresh_token")
+        self.assertEqual(access_token.grant_type, handler.refresh_grant_type)
         self.assertDictEqual(access_token.data, data)
         self.assertEqual(access_token.token, token)
         self.assertListEqual(access_token.scopes, scopes)
@@ -1478,6 +1484,59 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         self.assertDictContainsSubset(expected_headers, result.headers)
         self.assertEqual(json.dumps(expected_response_body), result.body)
 
+    @patch("time.time", mock_time)
+    def test_process_with_reissue(self):
+        client_id = "testclient"
+        data = {"additional": "data"}
+        expires_in = 600
+        scopes = []
+        token = "abcdefg"
+        refresh_token = "cefg"
+        expected_response_body = {"access_token": token,
+                                  "expires_in": expires_in,
+                                  "refresh_token": refresh_token,
+                                  "token_type": "Bearer"}
+        expected_headers = {"Content-Type": "application/json",
+                            "Cache-Control": "no-store", "Pragma": "no-cache"}
+
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+
+        response = Response()
+
+        scope_handler_mock = Mock(spec=Scope)
+        scope_handler_mock.scopes = scopes
+
+        token_data = {"access_token": token, "expires_in":expires_in, "token_type": "Bearer", "refresh_token":refresh_token}
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
+        token_generator_mock.refresh_expires_in = 1200
+
+        handler = RefreshTokenHandler(
+            access_token_store=access_token_store_mock,
+            client_authenticator=Mock(spec=ClientAuthenticator),
+            scope_handler=scope_handler_mock,
+            token_generator=token_generator_mock,
+            reissue_refresh_tokens=True)
+        handler.client = Client(identifier=client_id, secret="xyz")
+        handler.data = data
+        handler.refresh_grant_type = 'test_grant_type'
+
+        result = handler.process(request=Mock(spec=Request),
+                                 response=response, environ={})
+
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertEqual(access_token.client_id, client_id)
+        self.assertEqual(access_token.grant_type, handler.refresh_grant_type)
+        self.assertDictEqual(access_token.data, data)
+        self.assertEqual(access_token.token, token)
+        self.assertListEqual(access_token.scopes, scopes)
+        self.assertEqual(access_token.expires_at, 1600)
+
+        self.assertEqual(result, response)
+        self.assertDictContainsSubset(expected_headers, result.headers)
+        self.assertDictEqual(expected_response_body, json.loads(result.body))
+
+
     @patch("time.time", mock_time)
     def test_read_validate_params(self):
         client_id = "client"
@@ -1488,8 +1547,9 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         scopes = []
 
         access_token = AccessToken(client_id=client_id, token=original_token,
-                                   grant_type=RefreshToken.grant_type,
-                                   data=data, expires_at=1234, scopes=scopes)
+                                   grant_type="test_grant_type",
+                                   data=data, expires_at=1234, scopes=scopes,
+                                   refresh_expires_at=0)
 
         access_token_store_mock = Mock(AccessTokenStore)
         access_token_store_mock.fetch_by_refresh_token.return_value = access_token
@@ -1503,13 +1563,16 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [refresh_token]
 
+        token_generator_mock = Mock(expires_in={'test_grant_type':600})
+        token_generator_mock.refresh_expires_in = 0
+
         scope_handler_mock = Mock(spec=Scope)
 
         handler = RefreshTokenHandler(
             access_token_store=access_token_store_mock,
             client_authenticator=client_auth_mock,
             scope_handler=scope_handler_mock,
-            token_generator=Mock())
+            token_generator=token_generator_mock)
 
         handler.read_validate_params(request=request_mock)
 
@@ -1530,7 +1593,7 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         handler = RefreshTokenHandler(access_token_store=Mock(),
                                       client_authenticator=Mock(),
                                       scope_handler=Mock(),
-                                      token_generator=Mock())
+                                      token_generator=Mock(expires_in=600))
 
         with self.assertRaises(OAuthInvalidError) as expected:
             handler.read_validate_params(request_mock)
@@ -1559,7 +1622,7 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         handler = RefreshTokenHandler(access_token_store=access_token_store_mock,
                                       client_authenticator=client_auth_mock,
                                       scope_handler=Mock(),
-                                      token_generator=Mock())
+                                      token_generator=Mock(expires_in=600))
 
         with self.assertRaises(OAuthInvalidError) as expected:
             handler.read_validate_params(request_mock)
@@ -1570,12 +1633,13 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         self.assertEqual(e.explanation, "Invalid refresh token")
 
     @patch("time.time", mock_time)
-    def test_read_validate_params_expired_access_token(self):
+    def test_read_validate_params_expired_refresh_token(self):
         client_id = "abc"
         secret = "xyz"
 
         access_token_mock = Mock(spec=AccessToken)
-        access_token_mock.expires_at = 900
+        access_token_mock.grant_type = 'test_grant_type'
+        access_token_mock.refresh_expires_at = 900
 
         access_token_store_mock = Mock(spec=AccessTokenStore)
         access_token_store_mock.fetch_by_refresh_token.return_value = access_token_mock
@@ -1592,7 +1656,7 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
             access_token_store=access_token_store_mock,
             client_authenticator=client_auth_mock,
             scope_handler=Mock(),
-            token_generator=Mock())
+            token_generator=Mock(expires_in={'test_grant_type':600}))
 
         with self.assertRaises(OAuthInvalidError) as expected:
             handler.read_validate_params(request_mock)
@@ -1602,6 +1666,7 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         self.assertEqual(e.error, "invalid_request")
         self.assertEqual(e.explanation, "Invalid refresh token")
 
+
 class ClientCredentialsGrantTestCase(unittest.TestCase):
     def test_call(self):
         token_path = "token"
@@ -1680,8 +1745,7 @@ class ClientCredentialsHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.generate.return_value = token
-        token_generator_mock.expires_in = expires_in
-
+        token_generator_mock.expires_in = {ClientCredentialsGrant.grant_type:expires_in}
         handler = ClientCredentialsHandler(
             access_token_store=access_token_store_mock,
             client_authenticator=Mock(),
@@ -1716,7 +1780,7 @@ class ClientCredentialsHandlerTestCase(unittest.TestCase):
 
         token_generator_mock = Mock(spec=TokenGenerator)
         token_generator_mock.generate.return_value = token
-        token_generator_mock.expires_in = expires_in
+        token_generator_mock.expires_in = {ClientCredentialsGrant.grant_type:expires_in}
 
         handler = ClientCredentialsHandler(
             access_token_store=access_token_store_mock,
